/* 
	ADS1115_sample.c - 12/9/2013. Written by David Purdie as part of the openlabtools initiative
	Initiates and reads a single sample from the ADS1115 (without error handling)
*/

#include "satrotor.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h> // open
#include <sys/ioctl.h>
#include <unistd.h>
#include <inttypes.h>  // uint8_t, etc
#include <linux/i2c-dev.h> // I2C bus definitions
#include <math.h>
#include <sys/file.h>

#define ADWANDELFAKTOR 4.096/32767.0
#define SPANNUNGSTEILER (7500.0+3600.0)/3600.0

int main() {
  int band = -1;
  double faktor = 1.0;
  char buffer[100];
  int pol1,pol2,pol3,pol4;
  int noband = 0;

  printf( "Content-type: text/html\n\n");
  fflush(stdout);

  if( getenv("REQUEST_METHOD") != NULL && !strcmp( "GET", getenv("REQUEST_METHOD"))) {
     if( getenv("QUERY_STRING") != NULL) {
       strcpy( buffer, getenv("QUERY_STRING"));
       if( strstr( buffer, "band=")) {
         sscanf( buffer, "band=%d", &band);
       }
     }
  }

  switch( band) {  //Maximale Spannung fuer 100% Vorlaufleistung
    case 2:  faktor = 1.0;
             break;
    case 70: 
    case 23:
    default: faktor = 1.0;
  }

  wiringPiSetup();
  noband = digitalRead(RESET1) || digitalRead(RESET2) || digitalRead(ALARM1) ||
           digitalRead(ALARM2) || digitalRead(PTT1) || digitalRead(PTT2) || 
           digitalRead(POWER1) || digitalRead(POWER2);
  if( !noband) {
    printf( "KEIN BAND GEWÄHLT (ALLES AUS)");
    return 0;
  }

  int ADS_address = 0x48;	// Address of our device on the I2C bus
  int I2CFile;
  
  uint8_t writeBuf[3];		// Buffer to store the 3 bytes that we write to the I2C device
  uint8_t readBuf[2];		// 2 byte buffer to store the data read from the I2C device
  
  int16_t val;				// Stores the 16 bit value of our ADC conversion
  
  I2CFile = open("/dev/i2c-1", O_RDWR);		// Open the I2C device
  flock( I2CFile, LOCK_EX);
  
  ioctl(I2CFile, I2C_SLAVE, ADS_address);   // Specify the address of the I2C Slave to communicate with

// Kanal 0
	  
  // These three bytes are written to the ADS1115 to set the config register and start a conversion 
  writeBuf[0] = 1;		// This sets the pointer register so that the following two bytes write to the config register
  writeBuf[1] = 0b11000011;    	// This sets the 8 MSBs of the config register (bits 15-8) to 11000011
  writeBuf[2] = 0b11100011; 	// This sets the 8 LSBs of the config register (bits 7-0) to 00000011
  
  // Initialize the buffer used to read data from the ADS1115 to 0
  readBuf[0]= 0;		
  readBuf[1]= 0;
	  
  // Write writeBuf to the ADS1115, the 3 specifies the number of bytes we are writing,
  // this begins a single conversion
  write(I2CFile, writeBuf, 3);	

  // Wait for the conversion to complete, this requires bit 15 to change from 0->1
  while ((readBuf[0] & 0x80) == 0)	// readBuf[0] contains 8 MSBs of config register, AND with 10000000 to select bit 15
  {
	  read(I2CFile, readBuf, 2);	// Read the config register into readBuf
  }

  writeBuf[0] = 0;					// Set pointer register to 0 to read from the conversion register
  write(I2CFile, writeBuf, 1);
  
  read(I2CFile, readBuf, 2);		// Read the contents of the conversion register into readBuf

  val = readBuf[0] << 8 | readBuf[1];	// Combine the two bytes of readBuf into a single 16 bit result 
  
  printf("POWER %.0f%% ----\n", (float)val*ADWANDELFAKTOR*SPANNUNGSTEILER*faktor * (float)val*ADWANDELFAKTOR*SPANNUNGSTEILER*faktor );
	

//AD-Kanal 1


  // These three bytes are written to the ADS1115 to set the config register and start a conversion
  writeBuf[0] = 1;              // This sets the pointer register so that the following two bytes write to the config register
  writeBuf[1] = 0b11010011;     // This sets the 8 MSBs of the config register (bits 15-8) to 11000011
  writeBuf[2] = 0b11100011;     // This sets the 8 LSBs of the config register (bits 7-0) to 00000011

  // Initialize the buffer used to read data from the ADS1115 to 0
  readBuf[0]= 0;
  readBuf[1]= 0;

  // Write writeBuf to the ADS1115, the 3 specifies the number of bytes we are writing,
  // this begins a single conversion
  write(I2CFile, writeBuf, 3);

  // Wait for the conversion to complete, this requires bit 15 to change from 0->1
  while ((readBuf[0] & 0x80) == 0)      // readBuf[0] contains 8 MSBs of config register, AND with 10000000 to select bit 15
  {
          read(I2CFile, readBuf, 2);    // Read the config register into readBuf
  }

  writeBuf[0] = 0;                                      // Set pointer register to 0 to read from the conversion register
  write(I2CFile, writeBuf, 1);

  read(I2CFile, readBuf, 2);            // Read the contents of the conversion register into readBuf

  val = readBuf[0] << 8 | readBuf[1];   // Combine the two bytes of readBuf into a single 16 bit result

  //printf("PTT %f (V)   \n", (float)val*ADWANDELFAKTOR);        // Print the result to terminal, first convert from binary value to mV

  if( (float)val*ADWANDELFAKTOR < 0.5) printf( " TX ");
  else if( (float)val*ADWANDELFAKTOR > 1.76) printf( " RX "); 
       else  printf( " (PA IST AUS) ");


//AD-Kanal 2

  // These three bytes are written to the ADS1115 to set the config register and start a conversion
  writeBuf[0] = 1;              // This sets the pointer register so that the following two bytes write to the config register
  writeBuf[1] = 0b11100011;     // This sets the 8 MSBs of the config register (bits 15-8) to 11000011
  writeBuf[2] = 0b11100011;     // This sets the 8 LSBs of the config register (bits 7-0) to 00000011

  // Initialize the buffer used to read data from the ADS1115 to 0
  readBuf[0]= 0;
  readBuf[1]= 0;

  // Write writeBuf to the ADS1115, the 3 specifies the number of bytes we are writing,
  // this begins a single conversion
  write(I2CFile, writeBuf, 3);

  // Wait for the conversion to complete, this requires bit 15 to change from 0->1
  while ((readBuf[0] & 0x80) == 0)      // readBuf[0] contains 8 MSBs of config register, AND with 10000000 to select bit 15
  {
          read(I2CFile, readBuf, 2);    // Read the config register into readBuf
  }

  writeBuf[0] = 0;                                      // Set pointer register to 0 to read from the conversion register
  write(I2CFile, writeBuf, 1);

  read(I2CFile, readBuf, 2);            // Read the contents of the conversion register into readBuf

  val = readBuf[0] << 8 | readBuf[1];   // Combine the two bytes of readBuf into a single 16 bit result

  //printf("Voltage Reading %f (V) ALARM \n", (float)val*ADWANDELFAKTOR);  // Print the result to terminal, first convert from binary value to mV
  if( (float)val*ADWANDELFAKTOR > 0.5) printf( " ---- ALARM ");
  //else printf( " PA OK ");

  flock( I2CFile, LOCK_UN);	
  close(I2CFile);

  //Zugriff ueber WiringPi
  pol1 = digitalRead( POL1);
  pol2 = digitalRead( POL2);
  pol3 = digitalRead( POL3);
  pol4 = digitalRead( POL4);

  printf( " ---- ");
  if( pol1 == 0 && pol2 == 0) printf( "Polarisation H");
  if( pol1 == 1 && pol2 == 0) printf( "Polarisation V");
  if( pol1 == 0 && pol2 == 1) printf( "Polarisation R");
  if( pol1 == 1 && pol2 == 1) printf( "Polarisation L");
  
  return 0;

}

